window.addEventListener('dblclick', (me: MouseEvent) => {
    if (window.document.fullscreenEnabled) // Full screen facility supported...
        if (window.document.fullscreenElement === null) // Currently *NOT* in full screen mode
            window.document.documentElement.requestFullscreen();
        else window.document.exitFullscreen()
    else window.confirm("Full screen mode not available on this machine...");
});

interface Cloneable {
    clone(): any;
}

class Invalid_temperature_exception extends Error { // Inheriting from JavaScript root type for exceptions...
    constructor(private readonly _value, ...params) {
        // Pass remaining arguments to parent constructor:
        super(...params)
    }

// 'message' is an attribute inside 'Error' and thus cannot be overridden as a method here:
    public getMessage(): string {
        return this.constructor.name + ": " + this._value;
    }
}

enum Temperature_unit {
    Celsius = "°C", Fahrenheit = "°F", Kelvin = "°K"
}

class Temperature implements Cloneable {

    public static readonly Min: number = -273.15; // in Celsius
    private _value: number; // in Celsius
    // private readonly _step: number; // Automatically creates by constructor...

    public asCelsius(): number {
        return this._value;
    }

    public asFahrenheit(): number {
        return 9. / 5. * this._value + 32.;
    }

    public asKelvin(): number {
        return this._value - Temperature.Min;
    }

    public clone(): any {
        return Object.create(this);
    }

    public decrement(): never | void { // 'Temperature' instance method...
        this._value -= this._step;
        if (this._value < Temperature.Min) {
            throw new Invalid_temperature_exception(this._value);
        }
        console.assert(this._value >= Temperature.Min);
    }

    public increment(): void {
        this._value += this._step;
    }

    public equals(t: Temperature): boolean {
        return this._value == t._value;
    }

    public greaterThan(t: Temperature): boolean {
        return this._value > t._value;
    }

    public greaterOrEqualThan(t: Temperature): boolean {
        return this._value >= t._value;
    }

    public lessThan(t: Temperature): boolean {
        return this._value < t._value;
    }

    public lessOrEqualThan(t: Temperature): boolean {
        return this._value <= t._value;
    }

    /* Caution: 'private readonly' creates automatic field... */
    public constructor(value: number = 0., unit: Temperature_unit = Temperature_unit.Celsius, private readonly _step = 0.0001) {
        switch (unit) {
            case Temperature_unit.Celsius:
                this._value = value;
                break;
            case Temperature_unit.Fahrenheit:
                this._value = (value - 32.) * 5. / 9.;
                break;
            case Temperature_unit.Kelvin:
                this._value = value + Temperature.Min;
                break;
            default:
                throw new Invalid_temperature_exception("Illegal temperature unit");
        }
        if (this._value < Temperature.Min) {
            throw new Invalid_temperature_exception(this._value);
        }
    }

    public toString(unit: Temperature_unit): string {
        switch (unit) {
            case Temperature_unit.Celsius:
                return this.asCelsius() + Temperature_unit.Celsius;
                break;
            case Temperature_unit.Fahrenheit:
                return this.asFahrenheit() + Temperature_unit.Fahrenheit;
                break;
            case Temperature_unit.Kelvin:
                return this.asKelvin() + Temperature_unit.Kelvin;
                break;
            default:
                throw "Illegal temperature unit";
        }
    }
}

const main = () => { // 'window.onload = main;'
    const t = new Temperature(18.); // <=> 'new Temperature(18., Temperature_unit.Celsius)'
    window.confirm(t.toString(Temperature_unit.Fahrenheit));
    const t_ = t.clone();
    window.confirm(t_.toString(Temperature_unit.Celsius));
    const t__ = new Temperature(0., Temperature_unit.Kelvin);
    // try {
    //     t__.decrement();
    // } catch (ite: any) { // Eventually, no type checking...
    //     window.confirm(ite.getMessage());
    // }
    try {
        t__.decrement();
    } catch (ite: unknown) { // Signature is from TypeScript 4.x only!
        window.confirm((ite as Invalid_temperature_exception).getMessage()); // Cast is imposed by 'unknown'...
    }
}



